﻿using System;
using System.Text;

namespace ETHZ.C0284H.Calculator.CalculatorNode.Visitor
{
    internal class NodePrinter: IVisitor
    {
        // Buffer to store the intermediate result.
        private readonly StringBuilder _buffer;

        /// <summary>
        /// Constructor.
        /// </summary>
        public NodePrinter()
        {
            _buffer = new StringBuilder();
        }

        /// <summary>
        /// Printing result of last task.
        /// </summary>
        public string Result
        {
            get { return _buffer.ToString(); }
        }

        /// <summary>
        /// Visit an operand
        /// </summary>
        /// <param name="op">
        /// A <see cref="Operand"/>
        /// </param>
        public void VisitOperand(Operand op)
        {
            if (op != null)
            {
                _buffer.Insert(0, op.OperandValue + Environment.NewLine);
            }
        }

        /// <summary>
        /// Visit operator "+"
        /// </summary>
        /// <param name="op">
        /// A <see cref="PlusOperator"/>
        /// </param>
        public void VisitPlusOperator(PlusOperator op)
        {
            if (op != null)
            {
                _buffer.Insert(0, op + Environment.NewLine);
            }
        }

        /// <summary>
        /// Visit operator "-"
        /// </summary>
        /// <param name="op">
        /// A <see cref="MinusOperator"/>
        /// </param>
        public void VisitMinusOperator(MinusOperator op)
        {
            _buffer.Insert(0, op + Environment.NewLine);
        }

        /// <summary>
        /// Visit operator "*"
        /// </summary>
        /// <param name="op">
        /// A <see cref="TimesOperator"/>
        /// </param>
        public void VisitTimesOperator(TimesOperator op)
        {
            _buffer.Insert(0, op + Environment.NewLine);
        }

        /// <summary>
        /// Visit operator "/"
        /// </summary>
        /// <param name="op">
        /// A <see cref="DivideByOperator"/>
        /// </param>
        public void VisitDivideByOperator(DivideByOperator op)
        {
            _buffer.Insert(0, op + Environment.NewLine);
        }

    }
}
